/* 
 * AndSys 0.3.m.29 Copyright (C) 2012, 2013 Giulio Orru'
 * 
 * Licensed under the MIT license:
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package net.sourceforge.andsys;

import android.annotation.SuppressLint;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.ListAdapter;
import android.widget.ListView;
import android.widget.Spinner;
import android.widget.Toast;
import android.widget.AdapterView.OnItemSelectedListener;

// TODO: Issue: How to handle screen orientation change when progress dialog and background thread active?
// TODO: Details about Launchable and NOT Launchable App (for User)

public class ActivityApps extends net.sourceforge.andsys.Activity implements OnItemClickListener,
        OnItemSelectedListener {

    /**
     * to: Maintain the position of applications list when go to another application. and
     * Consistency data on change device orientation
     */
    private ListView mListView = null;

    /**
     * to Select Right Spinner
     */
    private Spinner mSpinnerOrderby = null;
    private Spinner mSpinnerFilters = null;

    /**
     * Called when the activity is first created.This is where you should do all of your normal
     * static set up: create views, bind data to lists, etc. This method also provides you with a
     * Bundle containing the activity's previously frozen state, if there was one. Always followed
     * by onStart().
     * 
     * @see android.app.Activity#onCreate(android.os.Bundle)
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // Set the activity content from a layout resource
        setContentView(R.layout.activity_apps);

        // List View useful in Progress Dialog
        mListView = (ListView) findViewById(android.R.id.list);

        // Spinner Orderby
        mSpinnerOrderby = (Spinner) findViewById(R.id.spinnerPackagesOrderby);
        if (mSpinnerOrderby != null) {

            // Create an ArrayAdapter using the string array and a spinner layout
            // - android.R.layout.simple_spinner_item is the default (align left)
            // - R.layout.spinner_item it's mine (align center)
            ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(this,
                    R.array.apps__orderby, R.layout.spinner_item);

            // Specify the layout to use when the list of choices appears
            adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);

            // Apply the adapter to the spinner
            mSpinnerOrderby.setAdapter(adapter);
            mSpinnerOrderby.setOnItemSelectedListener(this);
            // fix ERROR: Index out of bound exception
            if (State.orderby < 0 || State.orderby >= Apps.COUNT_ORDERBY) {
                State.orderby = 0;
            }
            mSpinnerOrderby.setSelection(State.orderby);
        }

        // Spinner Filters
        mSpinnerFilters = (Spinner) findViewById(R.id.spinnerPackagesFilters);
        if (mSpinnerFilters != null) {

            // Create an ArrayAdapter using the string array and a spinner layout
            ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(this,
                    R.array.apps__filters, R.layout.spinner_item);

            // Specify the layout to use when the list of choices appears
            adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);

            // Apply the adapter to the spinner
            mSpinnerFilters.setAdapter(adapter);
            mSpinnerFilters.setOnItemSelectedListener(this);
            // fix ERROR: Index out of bound exception
            if (State.filters < 0 || State.filters >= Apps.COUNT_FILTERS) {
                State.filters = 0;
            }
            mSpinnerFilters.setSelection(State.filters);
        }

    }

    @Override
    protected void onStart() {
        super.onStart();

        /*
         * check ready
         */
        if (State.apps == null) {
            appsLoading();
        } else if (State.apps.size() <= 0) {
            appsLoading();
        } else {
            /*
             * READY!
             */

            /*
             * Initialize list Adapter
             * 
             * Consistency data on change device orientation
             */
            {

                ListAdapter listAdapter = new AppAdapter(getBaseContext(), R.layout.list_item,
                        State.apps);
                mListView.setAdapter(listAdapter);
            }

            mListView.setClickable(true);
            mListView.setOnItemClickListener(this);
            mListView.setLongClickable(true);

            // Long Click Action
            mListView.setOnItemLongClickListener(new AdapterView.OnItemLongClickListener() {
                @Override
                public boolean onItemLongClick(AdapterView<?> parent, View v, int position, long id) {
                    String packageName = State.apps.get(position).getName();
                    String packageFullname = State.apps.get(position).getPackage();

                    /*
                     * Launch AndSys?!?!
                     */
                    if (packageFullname.equals(getText(R.string.app_package))) {
                        Toast toast = Toast.makeText(getBaseContext(),
                                getText(R.string.apps__launch_hey), Toast.LENGTH_LONG);
                        toast.show();
                    } else {

                        /*
                         * Launcher Activity Exists?
                         */
                        if (State.apps.get(position).getFlagLaunchable()) {
                            Toast toast = Toast.makeText(getBaseContext(),
                                    getText(R.string.apps__launch) + " \"" + packageName + "\"",
                                    Toast.LENGTH_LONG);
                            toast.show();
                            // Launch application
                            Intent intent = getPackageManager().getLaunchIntentForPackage(
                                    packageFullname);
                            startActivity(intent);
                            // garbage collection
                            intent = null;

                        } else {
                            Toast toast = Toast.makeText(getBaseContext(),
                                    getText(R.string.apps__unable_to_launch) + " \"" + packageName
                                            + "\"", Toast.LENGTH_LONG);
                            toast.show();
                        }
                    }
                    return true;
                }
            });
        }

    }

    /**
     * Called when the activity will start interacting with the user. At this point your activity is
     * at the top of the activity stack, with user input going to it. Always followed by onPause().
     */
    @Override
    protected void onResume() {
        super.onResume();

        /*
         * check ready
         */
        if (State.apps == null) {
            appsLoading();
        } else if (State.apps.size() <= 0) {
            appsLoading();
        } else {

            /*
             * READY!
             */

            // to: Maintain the position of applications list when go to another application.
            mListView.setSelectionFromTop(State.appsPosition, 0);
        }
    }

    @Override
    protected void onPause() {
        super.onPause();

        /*
         * check ready
         */
        if (State.apps != null) {
            if (State.apps.size() > 0) {

                /*
                 * READY!
                 */

                // to: Maintain the position of applications list when go to another application.
                int first = mListView.getFirstVisiblePosition();
                // BUT: if select the last app, at come back the last app maybe hide...
                // fixed:
                int last = mListView.getLastVisiblePosition();
                int count = mListView.getCount();

                // count-1 safe last
                if (last < count - 1) {
                    // first position
                    State.appsPosition = first;
                } else {
                    // last position
                    State.appsPosition = last;
                }
            }
        }

    }

    /*
     * Into ListView mListView
     * 
     * @see android.widget.AdapterView.OnItemClickListener#onItemClick(android.widget.AdapterView,
     * android.view.View, int, long)
     */
    @Override
    @SuppressLint("InlinedApi")
    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {

        /*
         * check ready
         */
        if (State.apps != null) {
            if (State.apps.size() > 0) {
                /*
                 * READY!
                 */

                String packageName = State.apps.get(position).getName();
                String packageFullname = State.apps.get(position).getPackage();
                Toast toast = Toast.makeText(getBaseContext(), getText(R.string.apps__details_of)
                        + " \"" + packageName + "\"", Toast.LENGTH_LONG);
                toast.show();

                // Launch State details/remmove
                Intent intent = new Intent();
                Uri intentPackageUri = Uri.parse("package:" + packageFullname);
                intent.setData(intentPackageUri);
                String intentPutExtraName;
                if (android.os.Build.VERSION.SDK_INT >= 9) {
                    intent.setAction(android.provider.Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
                    intent.addCategory(Intent.CATEGORY_DEFAULT);
                    intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                    intentPutExtraName = new String("package");
                } else {
                    intent.setAction(Intent.ACTION_VIEW);
                    intent.setClassName("com.android.settings",
                            "com.android.settings.InstalledAppDetails");
                    if (android.os.Build.VERSION.SDK_INT == 8) {
                        intentPutExtraName = new String("pkg");
                    } else {
                        intentPutExtraName = new String("com.android.settings.ApplicationPkgName");
                    }
                }
                intent.putExtra(intentPutExtraName, packageFullname);
                startActivity(intent);
                intent.removeExtra(intentPutExtraName);
                // garbage collection
                intent = null;
            }
        }
    }

    /*
     * Into Spinner Orderby and Filters
     * 
     * to: setOnItemSelectedListener(this) OnItemSelectedListener: OnItemSelectedListener
     */
    @Override
    public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {

        /*
         * check ready
         */
        if (State.apps != null) {
            if (State.apps.size() > 0) {
                /*
                 * READY!
                 */

                if (parent == mSpinnerOrderby) {

                    /*
                     * Spinner Orderby
                     */

                    // Set Order by.. (in the global scope)
                    if (position >= 0 && position < Apps.COUNT_ORDERBY) {
                        if (position != State.orderby) {

                            /*
                             * MUST Save Spinner selected option
                             */
                            State.orderby = position;

                            // to: Displaying progress bar when waiting
                            final ProgressDialog progressDialog = ProgressDialog.show(this,
                                    getText(R.string.apps__sorting),
                                    parent.getItemAtPosition(position).toString());
                            Handler handler = new Handler();
                            handler.postDelayed(new Runnable() {
                                @Override
                                public void run() {

                                    // --- IN BACKGROUND: BEGIN ---

                                    // Order by.. (in the global scope)
                                    Apps.appsSort();

                                    // The AdapterView does nothing more than manage visibility.
                                    // Adapter: View packages list (from the global scope)
                                    ListAdapter listAdapter = new AppAdapter(getBaseContext(),
                                            R.layout.list_item, State.apps);
                                    mListView.setAdapter(listAdapter);

                                    // --- IN BACKGROUND: END ---

                                    // Dismiss the progress dialog
                                    progressDialog.dismiss();
                                }
                            }, 1000);
                        }
                    }
                } else {
                    // (parent == mSpinnerFilters)

                    /*
                     * Spinner Filters
                     */

                    // Set Order by.. (in the global scope)
                    if (position >= 0 && position < Apps.COUNT_FILTERS) {
                        if (position != State.filters) {

                            /*
                             * MUST Save Spinner selected option
                             */
                            State.filters = position;

                            appsLoading();
                        }
                    }

                }

            }
        }
    }

    /*
     * OnItemSelectedListener: onNothingSelected
     */
    @Override
    public void onNothingSelected(AdapterView<?> arg0) {
        // TODO Auto-generated method stub
    }

    /*
     * Load data and assign mListView and listAdapter
     */
    public void appsLoading() {

        // to: Displaying progress bar when waiting
        final Context context = this.getBaseContext();

        final ProgressDialog progressDialog = ProgressDialog.show(this,
                getText(R.string.label__loading), getText(R.string.label__loading_text));
        Handler handler = new Handler();
        handler.postDelayed(new Runnable() {
            @Override
            public void run() {

                // --- IN BACKGROUND: BEGIN ---

                // Acquire Packages List (in the global scope) State
                Apps.apps(context);

                // Order by.. (in the global scope)
                Apps.appsSort();

                {

                    /*
                     * Set spinner selection
                     * 
                     * to: fix Apps spinner selection inconsistent when come back from another
                     * activity (apps is null)
                     */
                    if (State.filters < 0 || State.filters >= Apps.COUNT_FILTERS) {
                        State.filters = 0;
                    }
                    if (State.orderby < 0 || State.orderby >= Apps.COUNT_ORDERBY) {
                        State.orderby = 0;
                    }
                    mSpinnerFilters.setSelection(State.filters);
                    mSpinnerOrderby.setSelection(State.orderby);

                    /*
                     * Initialize list Adapter
                     * 
                     * Consistency data on change device orientation
                     */
                    ListAdapter listAdapter = new AppAdapter(getBaseContext(), R.layout.list_item,
                            State.apps);
                    mListView.setAdapter(listAdapter);
                }

                // --- IN BACKGROUND: END ---

                // Dismiss the progress dialog
                progressDialog.dismiss();

                // Restart the Activity with new Data
                onStart();
            }

        }, 1500);
    }

}
