/* 
 * AndSys 0.3.m.29 Copyright (C) 2012, 2013 Giulio Orru'
 * 
 * Licensed under the MIT license:
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package net.sourceforge.andsys;

import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

public class Apps {

    /**
     * Sort / order by
     * 
     * CHANGE BOTH (res)R.array.apps__orderby AND (src)Apps.java!!
     */
    public static final int ORDERBY_NAME = 0;
    public static final int ORDERBY_NAME_DESC = 1;
    public static final int ORDERBY_PACKAGE = 2;
    public static final int ORDERBY_PACKAGE_DESC = 3;
    public static final int ORDERBY_INSTALLED = 4;
    public static final int ORDERBY_INSTALLED_DESC = 5;
    public static final int ORDERBY_PERMISSIONS = 6;
    public static final int ORDERBY_PERMISSIONS_DESC = 7;
    public static final int COUNT_ORDERBY = 8;

    /**
     * Filters
     * 
     * CHANGE BOTH (res)R.array.apps__filters AND (src)Apps.java!!
     */
    public static final int FILTERS_LAUNCHABLE = 0; // First character As short Id
    public static final int FILTERS_NO_LAUNCHABLE = 1;
    public static final int FILTERS_SYSTEM = 2; // First character As short Id
    public static final int FILTERS_NO_SYSTEM = 3;
    public static final int FILTERS_ALL = 4;
    public static final int COUNT_FILTERS = 5;

    /*
     * Packages Activity Methods
     */

    public static final void apps(Context context) {
        State.apps = Apps.listFlags(context, State.filters);
    }

    /**
     * Sorting the packages
     */
    public static final void appsSort() {
        Comparator<App> pc;
        switch (State.orderby) {
        case Apps.ORDERBY_PERMISSIONS_DESC:
            pc = new App.ComparatorPermissions();
            pc = Collections.reverseOrder(pc);
            break;
        case Apps.ORDERBY_PERMISSIONS:
            pc = new App.ComparatorPermissions();
            break;
        case Apps.ORDERBY_INSTALLED_DESC:
            pc = new App.ComparatorInstalled();
            pc = Collections.reverseOrder(pc);
            break;
        case Apps.ORDERBY_INSTALLED:
            pc = new App.ComparatorInstalled();
            break;
        case Apps.ORDERBY_PACKAGE_DESC:
            pc = new App.ComparatorPackage();
            pc = Collections.reverseOrder(pc);
            break;
        case Apps.ORDERBY_PACKAGE:
            pc = new App.ComparatorPackage();
            break;
        case Apps.ORDERBY_NAME_DESC:
            pc = new App.ComparatorName();
            pc = Collections.reverseOrder(pc);
            break;
        default: // Apps.ORDERBY_NAME:
            pc = new App.ComparatorName();
            break;

        }
        // check if list is ready
        if (State.apps != null) {
            if (State.apps.size() > 0) {
                Collections.sort(State.apps, pc);
            }
        }
    }

    /**
     * Make a list of installed Packages (FILTERED!)
     * 
     * @param context
     *            ERROR with getApplicationContext()!!! USE getBaseContext()
     * @return List of packages
     */
    public static final List<App> listFlags(final Context context, int filters) {

        /*
         * List<ApplicationInfo> aiList = context.getPackageManager().getInstalledApplications(
         * PackageManager.GET_UNINSTALLED_PACKAGES); for (ApplicationInfo ai : aiList) { if
         * ((ai.flags & ApplicationInfo.FLAG_SYSTEM) != 1) {
         * results.add(list.get(n).loadLabel(pm).toString()); Log.w("Installed Applications",
         * list.get(n).loadLabel(pm).toString()); }
         * 
         * }
         */

        PackageManager pm = context.getPackageManager();
        /*
         * acquire package: A List of PackageInfo objects, one for each package that is installed on
         * the device. In the unlikely case of there being no installed packages, an empty list is
         * returned. If flag GET_UNINSTALLED_PACKAGES is set, a list of all applications including
         * those deleted with DONT_DELETE_DATA (partially installed apps with data directory) will
         * be returned.
         * 
         * PackageManager.GET_PERMISSIONS: PackageInfo flag: return information about permissions in
         * the package in permissions.
         */
        List<PackageInfo> piList = pm.getInstalledPackages(PackageManager.GET_UNINSTALLED_PACKAGES
                | PackageManager.GET_PERMISSIONS);
        List<App> packs = new ArrayList<App>();
        for (PackageInfo pi : piList) {
            App ap = new App(pm, pi);
            
            if (filters == Apps.FILTERS_LAUNCHABLE && ap.getFlagLaunchable()) {
                packs.add(ap);

            } else if (filters == Apps.FILTERS_NO_LAUNCHABLE && !ap.getFlagLaunchable()) {
                packs.add(ap);

            } else if (filters == Apps.FILTERS_SYSTEM && ap.getFlagSystem()) {
                packs.add(ap);

            } else if (filters == Apps.FILTERS_NO_SYSTEM && !ap.getFlagSystem()) {
                packs.add(ap);

            } else if (filters == Apps.FILTERS_ALL) {
                packs.add(ap);

            } else {
                // nothing
            }
        }

        return packs;
    }

}
